/*
 * This file is part of TechReborn, licensed under the MIT License (MIT).
 *
 * Copyright (c) 2017 TechReborn
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

package techreborn.client.container.builder;

import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.inventory.*;
import net.minecraft.item.ItemStack;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.apache.commons.lang3.Range;
import org.apache.commons.lang3.tuple.MutableTriple;
import org.apache.commons.lang3.tuple.Pair;
import reborncore.common.util.ItemUtils;
import techreborn.client.container.IRightClickHandler;

import java.util.ArrayList;
import java.util.List;
import java.util.function.Consumer;
import java.util.function.IntConsumer;
import java.util.function.IntSupplier;
import java.util.function.Predicate;

public class BuiltContainer extends Container {

	private final String name;

	private final Predicate<EntityPlayer> canInteract;
	private final List<Range<Integer>> playerSlotRanges;
	private final List<Range<Integer>> tileSlotRanges;

	private final ArrayList<MutableTriple<IntSupplier, IntConsumer, Short>> shortValues;
	private final ArrayList<MutableTriple<IntSupplier, IntConsumer, Integer>> integerValues;
	private List<Consumer<InventoryCrafting>> craftEvents;
	private Integer[] integerParts;

	public BuiltContainer(final String name, final Predicate<EntityPlayer> canInteract,
	                      final List<Range<Integer>> playerSlotRange,
	                      final List<Range<Integer>> tileSlotRange) {
		this.name = name;

		this.canInteract = canInteract;

		this.playerSlotRanges = playerSlotRange;
		this.tileSlotRanges = tileSlotRange;

		this.shortValues = new ArrayList<>();
		this.integerValues = new ArrayList<>();
	}

	public void addShortSync(final List<Pair<IntSupplier, IntConsumer>> syncables) {

		for (final Pair<IntSupplier, IntConsumer> syncable : syncables)
			this.shortValues.add(MutableTriple.of(syncable.getLeft(), syncable.getRight(), (short) 0));
		this.shortValues.trimToSize();
	}

	public void addIntegerSync(final List<Pair<IntSupplier, IntConsumer>> syncables) {

		for (final Pair<IntSupplier, IntConsumer> syncable : syncables)
			this.integerValues.add(MutableTriple.of(syncable.getLeft(), syncable.getRight(), 0));
		this.integerValues.trimToSize();
		this.integerParts = new Integer[this.integerValues.size()];
	}

	public void addCraftEvents(final List<Consumer<InventoryCrafting>> craftEvents) {
		this.craftEvents = craftEvents;
	}

	public void addSlot(final Slot slot) {
		this.func_75146_a(slot);
	}

	@Override
	public boolean func_75145_c(final EntityPlayer playerIn) {
		return this.canInteract.test(playerIn);
	}

	@Override
	public final void func_75130_a(final IInventory inv) {
		if (!this.craftEvents.isEmpty())
			this.craftEvents.forEach(consumer -> consumer.accept((InventoryCrafting) inv));
	}

	@Override
	public ItemStack func_184996_a(int slotId, int dragType, ClickType clickTypeIn, EntityPlayer player) {
		if (dragType == 1 && slotId > 0 && slotId < 1000) {
			Slot slot = this.field_75151_b.get(slotId);
			if (slot instanceof IRightClickHandler) {
				if (((IRightClickHandler) slot).handleRightClick(slot.getSlotIndex(), player, this)) {
					return ItemStack.field_190927_a;
				}
			}
		}
		return super.func_184996_a(slotId, dragType, clickTypeIn, player);
	}

	@Override
	public void func_75142_b() {
		super.func_75142_b();

		for (final IContainerListener listener : this.field_75149_d) {

			int i = 0;
			if (!this.shortValues.isEmpty())
				for (final MutableTriple<IntSupplier, IntConsumer, Short> value : this.shortValues) {
					final short supplied = (short) value.getLeft().getAsInt();
					if (supplied != value.getRight()) {

						listener.func_71112_a(this, i, supplied);
						value.setRight(supplied);
					}
					i++;
				}

			if (!this.integerValues.isEmpty())
				for (final MutableTriple<IntSupplier, IntConsumer, Integer> value : this.integerValues) {
					final int supplied = value.getLeft().getAsInt();
					if (supplied != value.getRight()) {

						listener.func_71112_a(this, i, supplied >> 16);
						listener.func_71112_a(this, i + 1, (short) (supplied & 0xFFFF));
						value.setRight(supplied);
					}
					i += 2;
				}
		}
	}

	@Override
	public void func_75132_a(final IContainerListener listener) {
		super.func_75132_a(listener);

		int i = 0;
		if (!this.shortValues.isEmpty())
			for (final MutableTriple<IntSupplier, IntConsumer, Short> value : this.shortValues) {
				final short supplied = (short) value.getLeft().getAsInt();

				listener.func_71112_a(this, i, supplied);
				value.setRight(supplied);
				i++;
			}

		if (!this.integerValues.isEmpty())
			for (final MutableTriple<IntSupplier, IntConsumer, Integer> value : this.integerValues) {
				final int supplied = value.getLeft().getAsInt();

				listener.func_71112_a(this, i, supplied >> 16);
				listener.func_71112_a(this, i + 1, (short) (supplied & 0xFFFF));
				value.setRight(supplied);
				i += 2;
			}

	}

	@SideOnly(Side.CLIENT)
	@Override
	public void func_75137_b(final int id, final int value) {

		if (id < this.shortValues.size()) {
			this.shortValues.get(id).getMiddle().accept((short) value);
			this.shortValues.get(id).setRight((short) value);
		} else if (id - this.shortValues.size() < this.integerValues.size() * 2) {

			if ((id - this.shortValues.size()) % 2 == 0)
				this.integerParts[(id - this.shortValues.size()) / 2] = value;
			else {
				this.integerValues.get((id - this.shortValues.size()) / 2).getMiddle().accept(
					(this.integerParts[(id - this.shortValues.size()) / 2] & 0xFFFF) << 16 | value & 0xFFFF);
			}
		}
	}

	@Override
	public ItemStack func_82846_b(final EntityPlayer player, final int index) {

		ItemStack originalStack = ItemStack.field_190927_a;

		final Slot slot = this.field_75151_b.get(index);

		if (slot != null && slot.func_75216_d()) {

			final ItemStack stackInSlot = slot.func_75211_c();
			originalStack = stackInSlot.func_77946_l();

			boolean shifted = false;

			for (final Range<Integer> range : this.playerSlotRanges)
				if (range.contains(index)) {

					if (this.shiftToTile(stackInSlot))
						shifted = true;
					break;
				}

			if (!shifted)
				for (final Range<Integer> range : this.tileSlotRanges)
					if (range.contains(index)) {
						if (this.shiftToPlayer(stackInSlot))
							shifted = true;
						break;
					}

			slot.func_75220_a(stackInSlot, originalStack);
			if (stackInSlot.func_190916_E() <= 0)
				slot.func_75215_d(ItemStack.field_190927_a);
			else
				slot.func_75218_e();
			if (stackInSlot.func_190916_E() == originalStack.func_190916_E())
				return ItemStack.field_190927_a;
			slot.func_190901_a(player, stackInSlot);
		}
		return originalStack;
	}

	protected boolean shiftItemStack(final ItemStack stackToShift, final int start, final int end) {
		boolean changed = false;
		if (stackToShift.func_77985_e()) {
			for (int slotIndex = start; stackToShift.func_190916_E() > 0 && slotIndex < end; slotIndex++) {
				final Slot slot = this.field_75151_b.get(slotIndex);
				final ItemStack stackInSlot = slot.func_75211_c();
				if (stackInSlot != ItemStack.field_190927_a && ItemUtils.isItemEqual(stackInSlot, stackToShift, true, true)
					&& slot.func_75214_a(stackToShift)) {
					final int resultingStackSize = stackInSlot.func_190916_E() + stackToShift.func_190916_E();
					final int max = Math.min(stackToShift.func_77976_d(), slot.func_75219_a());
					if (resultingStackSize <= max) {
						stackToShift.func_190920_e(0);
						stackInSlot.func_190920_e(resultingStackSize);
						slot.func_75218_e();
						changed = true;
					} else if (stackInSlot.func_190916_E() < max) {
						stackToShift.func_190918_g(max - stackInSlot.func_190916_E());
						stackInSlot.func_190920_e(max);
						slot.func_75218_e();
						changed = true;
					}
				}
			}
		}
		if (stackToShift.func_190916_E() > 0) {
			for (int slotIndex = start; stackToShift.func_190916_E() > 0 && slotIndex < end; slotIndex++) {
				final Slot slot = this.field_75151_b.get(slotIndex);
				ItemStack stackInSlot = slot.func_75211_c();
				if (stackInSlot == ItemStack.field_190927_a && slot.func_75214_a(stackToShift)) {
					final int max = Math.min(stackToShift.func_77976_d(), slot.func_75219_a());
					stackInSlot = stackToShift.func_77946_l();
					stackInSlot.func_190920_e(Math.min(stackToShift.func_190916_E(), max));
					stackToShift.func_190920_e(-stackInSlot.func_190916_E());
					slot.func_75215_d(stackInSlot);
					slot.func_75218_e();
					changed = true;
				}
			}
		}
		return changed;
	}

	private boolean shiftToTile(final ItemStack stackToShift) {
		for (final Range<Integer> range : this.tileSlotRanges)
			if (this.shiftItemStack(stackToShift, range.getMinimum(), range.getMaximum() + 1))
				return true;
		return false;
	}

	private boolean shiftToPlayer(final ItemStack stackToShift) {
		for (final Range<Integer> range : this.playerSlotRanges)
			if (this.shiftItemStack(stackToShift, range.getMinimum(), range.getMaximum() + 1))
				return true;
		return false;
	}

	public String getName() {
		return this.name;
	}
}
